@@ -8,25 +8,74 @@ import ntpath
 from .common_structs import *
 from .streams.SystemInfoStream import PROCESSOR_ARCHITECTURE
 
+class VirtualSegment:
+	def __init__(self, start, end, start_file_address):
+		self.start = start
+		self.end = end
+		self.start_file_address = start_file_address
+
+
+
+		self.data = None
+
+	def inrange(self, start, end):
+		return self.start <= start and end<= self.end
+
 class MinidumpBufferedMemorySegment:
-	def __init__(self, memory_segment, file_handle):
+	def __init__(self, memory_segment, file_handle, chunksize = 10*1024):
 		self.start_address = memory_segment.start_virtual_address
 		self.end_address = memory_segment.end_virtual_address
-
-		file_handle.seek(memory_segment.start_file_address)
-		self.data = file_handle.read(memory_segment.size)
+		self.total_size = memory_segment.end_virtual_address - memory_segment.start_virtual_address
+		self.start_file_address = memory_segment.start_file_address
+		self.chunksize = chunksize
+		self.chunks = []
 
 	def inrange(self, position):
-		return self.start_address <= position <= self.end_address
+		return self.start_address <= position < self.end_address
 
 	def remaining_len(self, position):
 		if not self.inrange(position):
 			return None
 		return self.end_address - position
 
+	def find(self, file_handle, pattern, startpos):
+		data = self.read(file_handle, 0, -1)
+		return data.find(pattern, startpos)
+
+	def read(self, file_handle, start, end):
+		if end is None:
+			file_handle.seek(self.start_file_address + start)
+			return file_handle.read(self.end_address - (self.start_file_address + start))
+
+		for chunk in self.chunks:
+			if chunk.inrange(start, end):
+				return chunk.data[start - chunk.start: end - chunk.start]
+
+		if self.total_size <= 2*self.chunksize:
+			chunksize = self.total_size
+			vs = VirtualSegment(0, chunksize, self.start_file_address)
+			file_handle.seek(self.start_file_address)
+			vs.data = file_handle.read(chunksize)
+			self.chunks.append(vs)
+			return vs.data[start - vs.start: end - vs.start]
+
+		chunksize = max((end-start), self.chunksize)
+		if start + chunksize > self.end_address:
+			chunksize = self.end_address - start
+
+		vs = VirtualSegment(start, start+chunksize, self.start_file_address + start)
+		file_handle.seek(vs.start_file_address)
+		vs.data = file_handle.read(chunksize)
+		self.chunks.append(vs)
+
+		return vs.data[start - vs.start: end - vs.start]
+
+
+
 class MinidumpBufferedReader:
-	def __init__(self, reader):
+	def __init__(self, reader, segment_chunk_size = 10*1024):
 		self.reader = reader
+		self.segment_chunk_size = segment_chunk_size
 		self.memory_segments = []
 
 		self.current_segment = None
@@ -46,7 +95,7 @@ class MinidumpBufferedReader:
 		# not in cache, check if it's present in memory space. if yes then create a new buffered memeory object, and copy data
 		for memory_segment in self.reader.memory_segments:
 			if memory_segment.inrange(requested_position):
-				newsegment = MinidumpBufferedMemorySegment(memory_segment, self.reader.file_handle)
+				newsegment = MinidumpBufferedMemorySegment(memory_segment, self.reader.file_handle, chunksize=self.segment_chunk_size)
 				self.memory_segments.append(newsegment)
 				self.current_segment = newsegment
 				self.current_position = requested_position
@@ -54,6 +103,9 @@ class MinidumpBufferedReader:
 
 		raise Exception('Memory address 0x%08x is not in process memory space' % requested_position)
 
+	def get_reader(self):
+		return self.reader
+
 	def seek(self, offset, whence = 0):
 		"""
 		Changes the current address to an offset of offset. The whence parameter controls from which position should we count the offsets.
@@ -111,9 +163,9 @@ class MinidumpBufferedReader:
 		Returns up to length bytes from the current memory segment
 		"""
 		t = self.current_position + length
-		if not self.current_segment.inrange(t):
+		if not self.current_segment.inrange(t - 1):
 			raise Exception('Would read over segment boundaries!')
-		return self.current_segment.data[self.current_position - self.current_segment.start_address :t - self.current_segment.start_address]
+		return self.current_segment.read(self.reader.file_handle, self.current_position - self.current_segment.start_address , t - self.current_segment.start_address)
 
 	def read(self, size = -1):
 		"""
@@ -128,15 +180,15 @@ class MinidumpBufferedReader:
 
 			old_new_pos = self.current_position
 			self.current_position = self.current_segment.end_address
-			return self.current_segment.data[old_new_pos - self.current_segment.start_address:]
+			return self.current_segment.read(self.reader.file_handle, old_new_pos - self.current_segment.start_address, None)
 
 		t = self.current_position + size
-		if not self.current_segment.inrange(t):
+		if not self.current_segment.inrange(t - 1):
 			raise Exception('Would read over segment boundaries!')
 
 		old_new_pos = self.current_position
 		self.current_position = t
-		return self.current_segment.data[old_new_pos - self.current_segment.start_address :t - self.current_segment.start_address]
+		return self.current_segment.read(self.reader.file_handle, old_new_pos - self.current_segment.start_address, t - self.current_segment.start_address)
 
 	def read_int(self):
 		"""
@@ -164,7 +216,7 @@ class MinidumpBufferedReader:
 		"""
 		Searches for a pattern in the current memory segment
 		"""
-		pos = self.current_segment.data.find(pattern)
+		pos = self.current_segment.find(self.reader.file_handle, pattern)
 		if pos == -1:
 			return -1
 		return pos + self.current_position
@@ -176,7 +228,7 @@ class MinidumpBufferedReader:
 		pos = []
 		last_found = -1
 		while True:
-			last_found = self.current_segment.data.find(pattern, last_found + 1)
+			last_found = self.current_segment.find(self.reader.file_handle, pattern, last_found + 1)
 			if last_found == -1:
 				break
 			pos.append(last_found + self.current_segment.start_address)
@@ -217,7 +269,7 @@ class MinidumpBufferedReader:
 			return self.read_uint()
 
 	def find_in_module(self, module_name, pattern, find_first = False, reverse_order = False):
-		t = self.reader.search_module(module_name, pattern, find_first = find_first, reverse_order = reverse_order)
+		t = self.reader.search_module(module_name, pattern, find_first = find_first, reverse_order = reverse_order, chunksize = self.segment_chunk_size)
 		return t
 
 
@@ -226,6 +278,10 @@ class MinidumpBufferedReader:
 class MinidumpFileReader:
 	def __init__(self, minidumpfile):
 		self.modules = minidumpfile.modules.modules
+		self.unloaded_modules = []
+		if minidumpfile.unloaded_modules is not None:
+			self.unloaded_modules = minidumpfile.unloaded_modules.modules
+
 		self.sysinfo = minidumpfile.sysinfo
 
 		if minidumpfile.memory_segments_64:
@@ -252,34 +308,48 @@ class MinidumpFileReader:
 		else:
 			raise Exception('Unknown processor architecture %s! Please fix and submit PR!' % self.sysinfo.ProcessorArchitecture)
 
-	def get_buffered_reader(self):
-		return MinidumpBufferedReader(self)
+	def get_handler(self):
+		return self.file_handle
+
+	def get_memory(self):
+		return self.memory_segments
+
+	def get_buffered_reader(self, segment_chunk_size = 10*1024):
+		return MinidumpBufferedReader(self, segment_chunk_size = segment_chunk_size)
 
 	def get_module_by_name(self, module_name):
 		for mod in self.modules:
+			if ntpath.basename(mod.name).lower().find(module_name.lower()) != -1:
+				return mod
+		return None
+
+	def get_unloaded_by_name(self, module_name):
+		for mod in self.unloaded_modules:
 			if ntpath.basename(mod.name).find(module_name) != -1:
 				return mod
 		return None
 
-	def search_module(self, module_name, pattern, find_first = False, reverse_order = False):
+	def search_module(self, module_name, pattern, find_first = False, reverse_order = False, chunksize = 10*1024):
 		mod = self.get_module_by_name(module_name)
 		if mod is None:
-			raise Exception('Could not find module! %s' % module_name)
-		
+			mod = self.get_unloaded_by_name(module_name)
+			if mod is None:
+				raise Exception('Could not find module! %s' % module_name)
+
 		needles = []
 		for ms in self.memory_segments:
 			if mod.baseaddress <= ms.start_virtual_address < mod.endaddress:
-				needles+= ms.search(pattern, self.file_handle, find_first = find_first)
+				needles+= ms.search(pattern, self.file_handle, find_first = find_first, chunksize = chunksize)
 				if len(needles) > 0 and find_first is True:
 					return needles
 
 
 		return needles
 
-	def search(self, pattern, find_first = False):
+	def search(self, pattern, find_first = False, chunksize = 10*1024):
 		t = []
 		for ms in self.memory_segments:
-			t+= ms.search(pattern, self.file_handle, find_first = find_first)
+			t+= ms.search(pattern, self.file_handle, find_first = find_first, chunksize = chunksize)
 
 		return t
 
